from webtest import TestApp as Client
import pytest
from jsonschema import Draft3Validator

from more.jsonschema import JsonSchemaApp, loader


def test_jsonschema():
    class User(object):
        def __init__(self, name=None, age=None):
            self.name = name
            self.age = age

    user_schema = {
        'type': 'object',
        'properties': {
            'name': {
                'type': 'string',
                'minLength': 3
            },
            'age': {
                'type': 'integer',
                'minimum': 10
            }
        },
        'required': ['name', 'age']
    }

    class App(JsonSchemaApp):
        pass

    user = User()

    @App.path(model=User, path='/')
    def get_user():
        return user

    @App.json(model=User, request_method='POST', load=loader(user_schema))
    def user_post(self, request, json):
        for key, value in json.items():
            setattr(self, key, value)

    c = Client(App())

    c.post_json('/', {'name': 'Somebody', 'age': 22})
    assert user.name == 'Somebody'
    assert user.age == 22

    r = c.post_json('/', {'name': 'Another'}, status=422)
    assert r.json == ["'age' is a required property"]

    r = c.post_json('/', {'name': 'Another', 'age': 8}, status=422)
    assert r.json == ['8 is less than the minimum of 10']

    r = c.post_json('/', {'name': 'An', 'age': 8}, status=422)
    assert r.json == [
        '8 is less than the minimum of 10',
        "'An' is too short"
    ]

def test_jsonschema_Draft3Validator():
    class User(object):
        def __init__(self, name=None, age=None):
            self.name = name
            self.age = age

    user_schema = {
        'type': 'object',
        'properties': {
            'name': {
                'type': 'string',
                'minLength': 3,
                'required': True
            },
            'age': {
                'type': 'integer',
                'minimum': 10,
                'required': True
            }
        }
    }

    class App(JsonSchemaApp):
        pass

    user = User()

    @App.path(model=User, path='/')
    def get_user():
        return user

    load = loader(user_schema, validator=Draft3Validator)

    @App.json(model=User, request_method='POST', load=load)
    def user_post(self, request, json):
        for key, value in json.items():
            setattr(self, key, value)

    c = Client(App())

    c.post_json('/', {'name': 'Somebody', 'age': 22})
    assert user.name == 'Somebody'
    assert user.age == 22

    r = c.post_json('/', {'name': 'Another'}, status=422)
    assert r.json == ["'age' is a required property"]

    r = c.post_json('/', {'name': 'Another', 'age': 8}, status=422)
    assert r.json == ['8 is less than the minimum of 10']

    r = c.post_json('/', {'name': 'An', 'age': 8}, status=422)
    assert r.json == [
        '8 is less than the minimum of 10',
        "'An' is too short"
    ]



import morepath

from .errors import ValidationError


class App(morepath.App):
    pass


@App.json(model=ValidationError)
def validation_error_default(self, request):
    @request.after
    def adjust_status(response):
        response.status = 422
    errors = []
    for error in self.errors:
        errors.append(error.message)

    return errors




import io
from setuptools import setup, find_packages

long_description = (
    io.open('README.rst', encoding='utf-8').read()
    + '\n\n' +
    io.open('CHANGES.rst', encoding='utf-8').read())

setup(name='more.jsonschema',
      version='0.1',
      description="JSON Schema support for Morepath",
      long_description=long_description,
      author="Henri Hulski",
      author_email="henri.hulski@gazeta.pl",
      keywords='morepath validation',
      license="BSD",
      url="https://github.com/morepath/more.jsonschema",
      namespace_packages=['more'],
      packages=find_packages(),
      include_package_data=True,
      zip_safe=False,
      classifiers=[
          'Intended Audience :: Developers',
          'License :: OSI Approved :: BSD License',
          'Programming Language :: Python :: 2.7',
          'Programming Language :: Python :: 3.4',
          'Programming Language :: Python :: 3.5',
      ],
      install_requires=[
          'morepath >= 0.18',
          'jsonschema'
      ],
      extras_require=dict(
          test=[
              'pytest >= 2.9.1',
              'pytest-remove-stale-bytecode',
              'webtest'
          ],
          coverage=[
              'pytest-cov',
          ],
          pep8=[
              'flake8',
              'pep8-naming',
          ],
      ))



from functools import partial

from jsonschema import Draft4Validator

from .errors import ValidationError


def load(schema, validator, request):
    validator.check_schema(schema)
    v = validator(schema)
    errors = sorted(v.iter_errors(request.json), key=lambda e: e.path)
    if errors:
        raise ValidationError(errors)

    return request.json


def loader(schema, validator=Draft4Validator):
    """Create a load function based on schema dict and IValidator class.

    :param schema: The schema to validate with.
    :param validator: The IValidator class that will be used to validate
        the instance. Default to ``Draft4Validator``.

    You can plug this ``load`` function into a json view.

    Returns a ``load`` function that takes a request JSON body
    and uses the schema to validate it. This function raises
    :class:`jsonschema.ValidationError` if validation is not successful.
    """
    return partial(load, schema, validator)
